/*

MetaTrader History Toolkit
example.cs
v1.1
2018-08-28
Copyright  2013-2018 Heinz Traub

*/

// Compatible with C# 4.0 specs.

using System;
using System.Text;
using System.Globalization;

// Type aliases. Scope is valid only in this source file.
using ReturnCode = System.UInt32;
using HCCHandle = System.UInt32;

class CSharpExample
{
	private static int Main(string[] args)
	{
		Console.WriteLine("MetaTrader History Toolkit - C# Example");
		Console.WriteLine("Copyright (C) 2013-2018 Heinz Traub");
		if(args.Length > 0)
		{
			HCCHandle file;
			ReturnCode rc = HCC.HCCOpenFile(args[0], out file);
			if(!CheckHCCError(rc))
				return 1;
			Console.WriteLine("\nFile: {0}", args[0]);
			
			StringBuilder symbol = new StringBuilder(10); // Should fit any symbol.
			rc = HCC.HCCGetSymbol(file, symbol);
			if(!CheckHCCError(rc))
				return 1;
			Console.WriteLine("Symbol: {0}", symbol);
			
			Console.WriteLine("Timeframe: 1 Minute");
			
			/* // Example: Overriding parsing options with a custom separator.
			HCC.HCCParsingOptions pops;
			pops.separator = HCC.HCC_RATES_SEPARATOR.Custom;
			pops.custom_separator = new byte[] {0x55, 0x23, 0x74, 0x08}; // Example custom separator. This is random and will not produce any results.
			pops.custom_length = 4;
			pops.fast_parse = false;
			pops.halt_corrupt = false;
			pops.ignore_corrupt = false;
			rc = HCC.HCCSetParsingOptions(file, pops);
			if(!CheckHCCError(rc))
				return 1;*/
			
			Console.Write("Loading history...");
			uint hlength;
			rc = HCC.HCCLoadHistory(file, Convert.ToUInt32(HCC.HCC_TIMEFRAME.M1), 0, 3, out hlength); // Load only the first 3 rates.
			if(!CheckHCCError(rc))
				return 1;
			Console.WriteLine("\nRates Loaded: {0}", hlength);
			if(hlength > 0)
			{
				Console.WriteLine("This program will display the first 3 rates (if available):");
				pause();
				HCC.HCCRates r;
				uint precision;
				rc = HCC.HCCGetPrecision(file, out precision);
				if(!CheckHCCError(rc))
					return 1;
				NumberFormatInfo price_format = new NumberFormatInfo();
				price_format.NumberDecimalSeparator = ".";
				price_format.NumberDecimalDigits = Convert.ToInt32(precision);
				
				for(uint i = 0; i < hlength; i++)
				{
					rc = HCC.HCCGetRates(file, i, out r);
					if(!CheckHCCError(rc))
						return 1;
					
					Console.WriteLine("\n[RATES #{0}]", i + 1);
					DateTime dt = new DateTime(1970, 1, 1).AddSeconds(r.time);
					Console.WriteLine("Time: {0:yyyy.MM.dd HH:mm}", dt);
					Console.WriteLine("Open: " + r.open.ToString(price_format));
					Console.WriteLine("High: " + r.high.ToString(price_format));
					Console.WriteLine("Low: " + r.low.ToString(price_format));
					Console.WriteLine("Close: " + r.close.ToString(price_format));
					Console.WriteLine("Volume: {0}", r.volume);
					Console.WriteLine("Spread: {0}", r.spread);
				}
				
				rc = HCC.HCCCloseFile(file);
				if(!CheckHCCError(rc))
					return 1;
				Console.WriteLine("\nDone!\n");
			}
			else
				Console.WriteLine("No rates found, halting.");
		}
		else // Show usage.
			Console.WriteLine("\nUsage: Drag and drop a valid HCC file to example.exe in Windows shell.\n");
		pause();
		return 0;
	}
	
	private static void pause()
	{
		Console.Write("Press any key to continue...");
		Console.ReadKey(true);
	}
	
	private static bool CheckHCCError(ReturnCode rc)
	{
		if(rc != HCC.HCC_SUCCESS)
		{
			StringBuilder message = new StringBuilder(128); // Should fit any error message.
			string err_str = "HCC Error ({0})";
			if(HCC.HCCGetErrorMessage(rc, 0, message) == HCC.HCC_SUCCESS)
				err_str += ": " + message;
			Console.WriteLine(err_str, rc);
			pause();
			return false;
		}
		else
			return true;
	}
}